/********************************************************************************
 *   testcpl -- control panel items enumerator.                                 *
 *                                                                              * 
 *   Copyright (C) 2004-2007 dci (dev@cpp.in)                                   *
 *                                                                              *
 *   This program is free software; you can redistribute it and/or              *
 *   modify it under the terms of the GNU General Public License version 2      *
 *   as published by the Free Software Foundation.                              *
 *                                                                              *
 *   This program is distributed in the hope that it will be useful,            *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
 *   General Public License version 2 for more details.                         *
 *                                                                              *
 *   You should have received a copy of the GNU General Public License          *
 *   version 2 along with this library; if not, write to the Free Software      *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 ********************************************************************************
 */


#include <windows.h>
#include <cpl.h>

void ShowInfo(const char *str, ...)
{
	va_list va;
	char buf[1024];
	va_start(va, str);
	vsprintf(buf, str, va);
	va_end(va);

	LPTSTR buf1;

	buf1 = (LPTSTR)malloc(sizeof(TCHAR) * (strlen(buf)+1));
	MultiByteToWideChar(CP_ACP, 0, buf, -1, buf1, strlen(buf)+1);

	MessageBox(0, buf1, TEXT("dci"), MB_ICONINFORMATION);
	free(buf1);
}

HICON GetPanelIcon ( const wchar_t * module, int panel )
{
	HICON ret = NULL;
	HMODULE cpl;
	if ( cpl = LoadLibrary ( module ) )
	{
		LONG ( * mCPlApplet ) ( HWND hwndCPL, UINT message, LPARAM lParam1, LPARAM lParam2 ) = NULL;
		if ( mCPlApplet = ( LONG ( * ) ( HWND, UINT, LPARAM, LPARAM ) ) GetProcAddress ( cpl, L"CPlApplet" ) )
		{
			if ( mCPlApplet ( NULL, CPL_INIT, 0, 0 ) )
			{
				NEWCPLINFO panelinfo;
				if ( mCPlApplet ( NULL, CPL_NEWINQUIRE, ( LPARAM ) panel, ( LPARAM ) &panelinfo ) == 0 )
				{
					ret = panelinfo.hIcon;
					mCPlApplet ( NULL, CPL_STOP, ( LPARAM ) panel, ( LPARAM ) &panelinfo );
				}
				mCPlApplet ( NULL, CPL_EXIT, 0, 0 );
			}
		}
		FreeLibrary ( cpl );
	}
	return ret;
}

void LaunchPanel ( const wchar_t * module, int panel )
{
	HMODULE cpl;
	if ( cpl = LoadLibrary ( module ) )
	{
		LONG ( * mCPlApplet ) ( HWND hwndCPL, UINT message, LPARAM lParam1, LPARAM lParam2 ) = NULL;
		if ( mCPlApplet = ( LONG ( * ) ( HWND, UINT, LPARAM, LPARAM ) ) GetProcAddress ( cpl, L"CPlApplet" ) )
		{
			if ( mCPlApplet ( NULL, CPL_INIT, 0, 0 ) )
			{
				NEWCPLINFO panelinfo;
				if ( mCPlApplet ( NULL, CPL_NEWINQUIRE, ( LPARAM ) panel, ( LPARAM ) &panelinfo ) == 0 )
				{
					mCPlApplet ( NULL, CPL_DBLCLK, ( LPARAM ) panel, ( LPARAM ) &panelinfo );
					mCPlApplet ( NULL, CPL_STOP, ( LPARAM ) panel, ( LPARAM ) &panelinfo );
				}
				mCPlApplet ( NULL, CPL_EXIT, 0, 0 );
			}
		}
		FreeLibrary ( cpl );
	}
}

void EnumPanels ( const wchar_t * module )
{
	HMODULE cpl;
	if ( cpl = LoadLibrary ( module ) )
	{
		LONG ( * mCPlApplet ) ( HWND hwndCPL, UINT message, LPARAM lParam1, LPARAM lParam2 ) = NULL;
		if ( mCPlApplet = ( LONG ( * ) ( HWND, UINT, LPARAM, LPARAM ) ) GetProcAddress ( cpl, L"CPlApplet" ) )
		{
			if ( mCPlApplet ( NULL, CPL_INIT, 0, 0 ) )
			{
				int NumPanels = mCPlApplet ( NULL, CPL_GETCOUNT, 0, 0 );

				for ( int i=0; i < NumPanels; i++ )
				{
					NEWCPLINFO panelinfo;
					if ( mCPlApplet ( NULL, CPL_NEWINQUIRE, ( LPARAM ) i, ( LPARAM ) &panelinfo ) == 0 )
					{
						
						if ( MessageBox ( 0, panelinfo.szName, module, MB_YESNO ) == IDYES ) LaunchPanel ( module, i );

						DestroyIcon ( panelinfo.hIcon );
						mCPlApplet ( NULL, CPL_STOP, ( LPARAM ) i, ( LPARAM ) &panelinfo );
					}
				}


				mCPlApplet(NULL,CPL_EXIT,0,0);
			}
		}
		FreeLibrary(cpl);
	}
}



int WINAPI WinMain ( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nShowCmd )
{
	WIN32_FIND_DATA FindData;
	HANDLE hFindFile;

	if ( INVALID_HANDLE_VALUE != ( hFindFile = FindFirstFile(L"\\Windows\\*.cpl", &FindData ) ) ) do
	{
		if ( ! ( FindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) ) EnumPanels ( FindData.cFileName );
	}
	while ( FindNextFile ( hFindFile, &FindData ) );
	
	if ( INVALID_HANDLE_VALUE != hFindFile ) FindClose ( hFindFile );

	return 0;
}